/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2015 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
	
/*
		MODIFIED by Hexabitz for BitzOS (BOS) V0.1.5 - Copyright (C) 2017-2018 Hexabitz
    All rights reserved
*/

/* Includes ------------------------------------------------------------------*/
#include "BOS.h"


/* Private variables ---------------------------------------------------------*/
uint8_t mode = 0, playLevel = 0, playerNote = 0, myNotesTotal = 0; uint8_t playerSeq[10] = {0}, mySeq[10] = {0};
uint32_t t0 = 0;

/* Embedded wave variables */
uint32_t *waveAdd[MAX_WAVES] =  {
	(uint32_t *) &waveByteCode_Play[0],
	(uint32_t *) &waveByteCode_Train[0],
	(uint32_t *) &waveByteCode_Correct[0],
	(uint32_t *) &waveByteCode_Wrong[0],
};

const uint32_t waveLength[MAX_WAVES] =  {
	WAVEBYTECODE_PLAY_LENGTH,
	WAVEBYTECODE_TRAIN_LENGTH,
	WAVEBYTECODE_CORRECT_LENGTH,
	WAVEBYTECODE_WRONG_LENGTH,
};

const uint32_t waveResolution[MAX_WAVES] =  {
	WAVEBYTECODE_PLAY_BPS,
	WAVEBYTECODE_TRAIN_BPS,
	WAVEBYTECODE_CORRECT_BPS,
	WAVEBYTECODE_WRONG_BPS,
};

const uint32_t waveRate[MAX_WAVES] =  {
	WAVEBYTECODE_PLAY_SPS,
	WAVEBYTECODE_TRAIN_SPS,
	WAVEBYTECODE_CORRECT_SPS,
	WAVEBYTECODE_WRONG_SPS,
};

const char *waveName[MAX_WAVES] =  {		// Note wave name must be lower case
	"play",
	"train",
	"correct",
	"wrong",
};

/* Private function prototypes -----------------------------------------------*/
void playTheButtonNote(uint8_t port);
void generateSequence(uint8_t level);

/* Main functions ------------------------------------------------------------*/

int main(void)
{


  /* MCU Configuration----------------------------------------------------------*/

  /* Reset all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all user peripherals */

	/* Initialize BitzOS */
	BOS_Init();

  /* Call init function for freertos objects (in freertos.c) */
  MX_FREERTOS_Init();

  /* Start scheduler */
  osKernelStart();
  
  /* We should never get here as control is now taken by the scheduler */

  /* Infinite loop */
  while (1) {


  }
}

/*-----------------------------------------------------------*/

/* FrontEndTask function */
void FrontEndTask(void * argument)
{
	/* Define the push buttons */
	AddPortButton(MOMENTARY_NO, P1);
	AddPortButton(MOMENTARY_NO, P2);
	AddPortButton(MOMENTARY_NO, P3);
	AddPortButton(MOMENTARY_NO, P4);
	AddPortButton(MOMENTARY_NO, P5);
	
	/* Define button events */
	SetButtonEvents(P1, 1, 0, 0, 0, 0, 0, 0, 0);
	SetButtonEvents(P2, 1, 0, 0, 0, 0, 0, 0, 0);
	SetButtonEvents(P3, 1, 0, 0, 0, 0, 0, 0, 0);
	SetButtonEvents(P4, 1, 0, 0, 0, 0, 0, 0, 0);
	SetButtonEvents(P5, 1, 0, 2, 0, 0, 0, 0, 0);	
	
	/* A little Pac-Man for a startup tune! */
	B2(0.125); B3(0.0625); B2(0.125); B3(0.0625);
	C3(0.125); C4(0.0625); C3(0.125); C4(0.0625); 
	B2(0.125); B3(0.0625); B2(0.125); B3(0.0625);
	Fsharp3(0.125); Gsharp3(0.125); Asharp3(0.125); B3(0.125);
	
	/* Get starting time in ms */
	t0 = HAL_GetTick();
	
  /* Infinite loop */
  for(;;) {
		
		if (mode == 1 && (HAL_GetTick()-t0) > 10000)		// Play mode and timer has expired
		{
			/* Check the player sequence */
			for(uint8_t i=0 ; i<10 ; i++)
      {
				if (mySeq[i] != 0 && playerSeq[i] != mySeq[i])	// Oops.. Player mistake
				{
					PlayWave("wrong", 1, 0);	
					IND_blink(500);	
					Delay_ms(1000);					
					goto wrongResultExit;
				}
      }
			
			/* Correct result! Fantastic.. Increase difficulty */	
			if (mySeq[0] != 0)				// Make sure there's a challenge tune already!	
			{				
				PlayWave("correct", 1, 0);
				IND_blink(100); Delay_ms(100); IND_blink(100); Delay_ms(100); IND_blink(100); Delay_ms(100);
				++playLevel;
				Delay_ms(1000);
			}
			
wrongResultExit:				
			/* Clear both sequences */
			for(uint8_t n=0 ; n<10 ; n++) {
				playerSeq[n] = mySeq[n] = 0;
			}				
			playerNote = 0;
			
			/* Generate a new sequence and wait a bit */
			generateSequence(playLevel);
			Delay_ms(1000);
			
			/* Reset the timer */
			t0 = HAL_GetTick();
		}
	}
}

void buttonClickedCallback(uint8_t port)
{    
	playTheButtonNote(port);
	playerSeq[playerNote++] = port;
	/* Reset timer to 8 seconds if player entered the correct number of notes */
	if (playerNote == myNotesTotal) 
		t0 = HAL_GetTick()-8000;
}

void buttonPressedForXCallback(uint8_t port, uint8_t eventType)
{    
	if (port == P5 && eventType == PRESSED_FOR_X1_SEC) 
	{
		if (mode == 0) {			// Train
			mode = 1;
			PlayWave("play", 1, 0);
			t0 = HAL_GetTick()-8000;				// Reset timer to -8 seconds
		} else if (mode == 1) {	// Play
			mode = 0;
			PlayWave("train", 1, 0);		
		}
		resetButtonEvent(P5, PRESSED_FOR_X1_SEC);			// This is a latching event so we need to reset it manually
		IND_blink(300);
	}
}

/* Play the musical note associated with this button */
void playTheButtonNote(uint8_t port)
{
	switch (port)
  {
  	case P3 :	C5(0.25);	break;
  	case P4 :	D5(0.25);	break;
		case P5 :	E5(0.25);	break;
		case P1 :	F5(0.25);	break;
		case P2 :	G5(0.25);	break;
  	default:
  		break;
  }		
}

/* Generate a new musical tune */
void generateSequence(uint8_t level)
{
	uint8_t levelDivided = level / 5;
	uint8_t note = 0, j;
	
	for(j=0 ; j<(2+levelDivided) ; j++)
	{
		note = 1+rand()%5;				// Pick a random note from 1 to 5 (corresponding to the buttons)
		playTheButtonNote(note);
		mySeq[j] = note;
		Delay_ms(note*200/(levelDivided+1));						// some random between notes based on difficulty level
	}		
	myNotesTotal = j;
}

/*-----------------------------------------------------------*/

/************************ (C) COPYRIGHT HEXABITZ *****END OF FILE****/
